@@ -7,7 +7,7 @@ from slapp_py.core_classes.division import Division
 from slapp_py.core_classes.player import Player
 from slapp_py.core_classes.skill import Skill
 from slapp_py.core_classes.team import Team
-from slapp_py.helpers.sources_helper import attempt_link_source
+from slapp_py.slapp_runner.simple_source import SimpleSource
 
 
 class SlappResponseObject:
@@ -32,11 +32,9 @@ class SlappResponseObject:
                      "Item2": "Item2" in tup}
                 matched_players_for_teams[team_id].append(player_tuple_for_team)
 
-        sources: Dict[str, str] = {}
-
-        for source_id in response.get("Sources"):
-            source_name = response.get("Sources")[source_id]
-            sources[source_id] = source_name
+        sources: Dict[str, SimpleSource] = {}
+        for source_id, source_name in response.get("Sources").items():
+            sources[source_id] = SimpleSource(id=source_id, name=source_name)
 
         for player_id in response.get("PlacementsForPlayers"):
             placements_for_players[player_id.__str__()] = {}
@@ -133,29 +131,42 @@ class SlappResponseObject:
         players = self.get_players_in_team(team_guid, include_ex_players)
         return {player: player.skill for player in players}
 
-    def get_sources_for_player(self, p: Player) -> Dict[str, List[Bracket]]:
-        """
-        Gets the sources for the specified player, keyed by source id.
-        """
-        return self.placements_for_players.get(p.guid.__str__(), {})
+    def get_simple_sources(self, sources: Union[Player, Team, List[UUID]]) -> List[SimpleSource]:
+        """Get the SimpleSource list for the specified sources/team/player."""
+        if isinstance(sources, Player):
+            sources = sources.sources
+        elif isinstance(sources, Team):
+            sources = sources.sources
+        if not isinstance(sources, list):
+            sources = [sources]
 
-    def get_source_names_for_player(self, p: Player) -> List[str]:
-        """
-        Gets the sources for the specified player in name form.
-        """
         result = []
-        for source in p.sources:
+        for source in sources:
             from slapp_py.core_classes.builtins import BuiltinSource
             if source == BuiltinSource.guid:
-                result.append("(builtin)")
+                result.append(SimpleSource(id=BuiltinSource.guid, name=BuiltinSource.name))
             else:
-                name = self.sources.get(source.__str__(), None)
-                if not name:
+                simple_source = self.sources.get(source.__str__(), None)
+                if not simple_source:
                     logging.error(f"Source was not specified in JSON: {source}")
                 else:
-                    result.append(name)
+                    result.append(simple_source)
         return result
 
+    def get_grouped_sources(self, sources: Union[Player, Team, List[UUID]]) -> Dict[str, List[SimpleSource]]:
+        """Group the SimpleSource list for the specified sources/team/player by tourney name."""
+        simple_sources = self.get_simple_sources(sources)
+        result = {}
+        for simple_source in simple_sources:
+            result.setdefault(simple_source.tournament_name or '', []).append(simple_source)
+        return result
+
+    def get_grouped_sources_text(self, sources: Union[Player, Team, List[UUID]]) -> List[str]:
+        """Group the SimpleSource list for the specified sources/team/player by tourney name."""
+        groups = self.get_grouped_sources(sources)
+        return [f"{tourney + ': ' if tourney else ''}{', '.join([simple_source.get_linked_date_display() for simple_source in simple_sources])}"
+                for tourney, simple_sources in groups.items()]
+
     def get_brackets_for_player(self, p: Player, source_id: str) -> List[Bracket]:
         """
         Gets the brackets for the specified player for the source specified.
@@ -168,15 +179,15 @@ class SlappResponseObject:
         where the specified player has come first.
         """
         result = []
-        sources = self.get_sources_for_player(p)
-        for source_id in sources:
-            brackets = self.get_brackets_for_player(p, source_id)
+        sources = self.get_simple_sources(p)
+        for simple_source in sources:
+            brackets = self.get_brackets_for_player(p, simple_source.id)
             for bracket in brackets:
                 if 1 in bracket.placements.players_by_placement:
                     first_place_ids = [player_id.__str__() for player_id in
                                        bracket.placements.players_by_placement[1]]
                     if p.guid.__str__() in first_place_ids:
-                        result.append(bracket.name + ' in ' + attempt_link_source(self.sources[source_id]))
+                        result.append(bracket.name + ' in ' + simple_source.url)
 
         return result
 
@@ -187,17 +198,18 @@ class SlappResponseObject:
         """
         result = []
         low_ink_sources = [s for s in self.placements_for_players.get(p.guid.__str__(), []) if
-                           "low-ink-" in self.sources.get(s, s)]
+                           "low-ink-" in self.sources.get(s, s).name]
 
         for source_id in low_ink_sources:
-            source_name = self.sources[source_id]
+            placements_for_source = self.placements_for_players[p.guid.__str__()][source_id]
+            source_name = self.sources[source_id].name
 
             # Take only the brackets useful to us
             # i.e. Alpha, Beta, Gamma, and previous Top Cuts.
             # Plus the bracket must have had placements in it (a winning team)
             # We can't rely on placements that don't have one of these brackets as it may indicate that the
             # team dropped rather than was unplaced, and so is not in accordance with skill
-            brackets = [b for b in self.placements_for_players[p.guid.__str__()][source_id] if
+            brackets = [b for b in placements_for_source if
                         (
                                 "Alpha" in b.name or
                                 "Beta" in b.name or
